@@ -143,7 +143,7 @@ void mod_gm_decrypt(char ** decrypted, char * text, int mode) {
 
     /* first decode from base64 */
     size_t bsize = base64_decode(text, buffer, input_size);
-    test = gm_strndup(buffer, 5);
+    test = gm_strndup((char *)buffer, 5);
     if(mode == GM_ENCODE_AND_ENCRYPT || (mode == GM_ENCODE_ACCEPT_ALL && strcmp(test, "type="))) {
         /* then decrypt */
         mod_gm_aes_decrypt(decrypted, buffer, bsize);
@@ -233,6 +233,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->debug_level        = GM_LOG_INFO;
     opt->perfdata           = GM_DISABLED;
     opt->perfdata_mode      = GM_PERFDATA_OVERWRITE;
+    opt->perfdata_send_all  = GM_DISABLED;
     opt->use_uniq_jobs      = GM_ENABLED;
     opt->do_hostchecks      = GM_ENABLED;
     opt->route_eventhandler_like_checks = GM_DISABLED;
@@ -285,6 +286,9 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->dupserver_num         = 0;
     for(i=0;i<GM_LISTSIZE;i++)
         opt->dupserver_list[i] = NULL;
+    opt->perfdata_queues_num = 0;
+    for(i=0;i<GM_LISTSIZE;i++)
+        opt->perfdata_queues_list[i] = NULL;
     opt->hostgroups_num     = 0;
     for(i=0;i<GM_LISTSIZE;i++)
         opt->hostgroups_list[i] = NULL;
@@ -305,6 +309,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     }
     opt->exports_count = 0;
     opt->restrict_path_num      = 0;
+    opt->gearman_connection_timeout = -1;
     for(i=0;i<GM_LISTSIZE;i++)
         opt->restrict_path[i] = NULL;
 
@@ -379,15 +384,35 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     }
 
     /* perfdata */
-    else if (   !strcmp( key, "perfdata" ) ) {
+    else if ( !strcmp(key, "perfdata") && strlen(value) ) {
         opt->set_queues_by_hand++;
-        opt->perfdata = parse_yes_or_no(value, GM_ENABLED);
-        /* perfdata override to dump all performance values */
-        if(value != NULL) {
-            lc(value);
-            if(!strcmp( value, "all" ))
-                opt->perfdata = GM_PERFDATA_ALL;
+        opt->perfdata = parse_yes_or_no(value, -1);
+
+        if (opt->perfdata == GM_ENABLED) {
+            //bool, use default queue
+            opt->perfdata_queues_list[opt->perfdata_queues_num] = gm_strdup(GM_PERFDATA_QUEUE);
+            opt->perfdata_queues_num++;
         }
+        else if (opt->perfdata == -1) {
+            //not a bool, use value string as queue name(s)
+            opt->perfdata = GM_ENABLED;
+
+            char *values, *values_original;
+            values_original = values = gm_strdup(value);
+            char *name;
+            while ((name = strsep(&values, ",")) != NULL) {
+                opt->perfdata_queues_list[opt->perfdata_queues_num] = gm_strdup(name);
+                opt->perfdata_queues_num++;
+            }
+            free(values_original);
+        }
+        return(GM_OK);
+    }
+
+    /* perfdata_send_all */
+    else if ( !strcmp(key, "perfdata_send_all") ) {
+        /* perfdata override to dump all performance values */
+        opt->perfdata_send_all = parse_yes_or_no(value, GM_DISABLED);
         return(GM_OK);
     }
 
@@ -894,6 +919,11 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         opt->restrict_command_characters = gm_strdup(value);
     }
 
+    /* timeout while connecting to gearmand server*/
+    else if ( !strcmp( key, "gearman_connection_timeout" ) ) {
+        opt->gearman_connection_timeout = atoi( value );
+    }
+
     else {
         gm_log( GM_LOG_ERROR, "unknown option '%s'\n", key );
     }
@@ -998,7 +1028,8 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
     if(mode == GM_NEB_MODE) {
         gm_log( GM_LOG_DEBUG, "queue by cust var:               %s\n", opt->queue_cust_var == NULL ? "no" : opt->queue_cust_var);
         gm_log( GM_LOG_DEBUG, "debug result:                    %s\n", opt->debug_result == GM_ENABLED ? "yes" : "no");
-        //gm_log( GM_LOG_DEBUG, "result_worker:                   %d\n", opt->result_workers);
+        if(opt->result_workers != 1)
+            gm_log( GM_LOG_DEBUG, "result_worker:                   %d\n", opt->result_workers);
         gm_log( GM_LOG_DEBUG, "do_hostchecks:                   %s\n", opt->do_hostchecks == GM_ENABLED ? "yes" : "no");
         gm_log( GM_LOG_DEBUG, "route_eventhandler_like_checks:  %s\n", opt->route_eventhandler_like_checks == GM_ENABLED ? "yes" : "no");
     }
@@ -1363,275 +1394,6 @@ char *replace_str(const char *str, const char *old, const char *new) {
     return ret;
 }
 
-
-/* return human readable name for neb type */
-char * nebtype2str(int i) {
-    switch(i) {
-        case 0:
-            return gm_strdup("NEBTYPE_NONE"); break;
-        case 1:
-            return gm_strdup("NEBTYPE_HELLO"); break;
-        case 2:
-            return gm_strdup("NEBTYPE_GOODBYE"); break;
-        case 3:
-            return gm_strdup("NEBTYPE_INFO"); break;
-        case 100:
-            return gm_strdup("NEBTYPE_PROCESS_START"); break;
-        case 101:
-            return gm_strdup("NEBTYPE_PROCESS_DAEMONIZE"); break;
-        case 102:
-            return gm_strdup("NEBTYPE_PROCESS_RESTART"); break;
-        case 103:
-            return gm_strdup("NEBTYPE_PROCESS_SHUTDOWN"); break;
-        case 104:
-            return gm_strdup("NEBTYPE_PROCESS_PRELAUNCH"); break;
-        case 105:
-            return gm_strdup("NEBTYPE_PROCESS_EVENTLOOPSTART"); break;
-        case 106:
-            return gm_strdup("NEBTYPE_PROCESS_EVENTLOOPEND"); break;
-        case 200:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_ADD"); break;
-        case 201:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_REMOVE"); break;
-        case 202:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_EXECUTE"); break;
-        case 203:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_DELAY"); break;
-        case 204:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_SKIP"); break;
-        case 205:
-            return gm_strdup("NEBTYPE_TIMEDEVENT_SLEEP"); break;
-        case 300:
-            return gm_strdup("NEBTYPE_LOG_DATA"); break;
-        case 301:
-            return gm_strdup("NEBTYPE_LOG_ROTATION"); break;
-        case 400:
-            return gm_strdup("NEBTYPE_SYSTEM_COMMAND_START"); break;
-        case 401:
-            return gm_strdup("NEBTYPE_SYSTEM_COMMAND_END"); break;
-        case 500:
-            return gm_strdup("NEBTYPE_EVENTHANDLER_START"); break;
-        case 501:
-            return gm_strdup("NEBTYPE_EVENTHANDLER_END"); break;
-        case 600:
-            return gm_strdup("NEBTYPE_NOTIFICATION_START"); break;
-        case 601:
-            return gm_strdup("NEBTYPE_NOTIFICATION_END"); break;
-        case 602:
-            return gm_strdup("NEBTYPE_CONTACTNOTIFICATION_START"); break;
-        case 603:
-            return gm_strdup("NEBTYPE_CONTACTNOTIFICATION_END"); break;
-        case 604:
-            return gm_strdup("NEBTYPE_CONTACTNOTIFICATIONMETHOD_START"); break;
-        case 605:
-            return gm_strdup("NEBTYPE_CONTACTNOTIFICATIONMETHOD_END"); break;
-        case 700:
-            return gm_strdup("NEBTYPE_SERVICECHECK_INITIATE"); break;
-        case 701:
-            return gm_strdup("NEBTYPE_SERVICECHECK_PROCESSED"); break;
-        case 702:
-            return gm_strdup("NEBTYPE_SERVICECHECK_RAW_START"); break;
-        case 703:
-            return gm_strdup("NEBTYPE_SERVICECHECK_RAW_END"); break;
-        case 704:
-            return gm_strdup("NEBTYPE_SERVICECHECK_ASYNC_PRECHECK"); break;
-        case 800:
-            return gm_strdup("NEBTYPE_HOSTCHECK_INITIATE"); break;
-        case 801:
-            return gm_strdup("NEBTYPE_HOSTCHECK_PROCESSED"); break;
-        case 802:
-            return gm_strdup("NEBTYPE_HOSTCHECK_RAW_START"); break;
-        case 803:
-            return gm_strdup("NEBTYPE_HOSTCHECK_RAW_END"); break;
-        case 804:
-            return gm_strdup("NEBTYPE_HOSTCHECK_ASYNC_PRECHECK"); break;
-        case 805:
-            return gm_strdup("NEBTYPE_HOSTCHECK_SYNC_PRECHECK"); break;
-        case 900:
-            return gm_strdup("NEBTYPE_COMMENT_ADD"); break;
-        case 901:
-            return gm_strdup("NEBTYPE_COMMENT_DELETE"); break;
-        case 902:
-            return gm_strdup("NEBTYPE_COMMENT_LOAD"); break;
-        case 1000:
-            return gm_strdup("NEBTYPE_FLAPPING_START"); break;
-        case 1001:
-            return gm_strdup("NEBTYPE_FLAPPING_STOP"); break;
-        case 1100:
-            return gm_strdup("NEBTYPE_DOWNTIME_ADD"); break;
-        case 1101:
-            return gm_strdup("NEBTYPE_DOWNTIME_DELETE"); break;
-        case 1102:
-            return gm_strdup("NEBTYPE_DOWNTIME_LOAD"); break;
-        case 1103:
-            return gm_strdup("NEBTYPE_DOWNTIME_START"); break;
-        case 1104:
-            return gm_strdup("NEBTYPE_DOWNTIME_STOP"); break;
-        case 1200:
-            return gm_strdup("NEBTYPE_PROGRAMSTATUS_UPDATE"); break;
-        case 1201:
-            return gm_strdup("NEBTYPE_HOSTSTATUS_UPDATE"); break;
-        case 1202:
-            return gm_strdup("NEBTYPE_SERVICESTATUS_UPDATE"); break;
-        case 1203:
-            return gm_strdup("NEBTYPE_CONTACTSTATUS_UPDATE"); break;
-        case 1300:
-            return gm_strdup("NEBTYPE_ADAPTIVEPROGRAM_UPDATE"); break;
-        case 1301:
-            return gm_strdup("NEBTYPE_ADAPTIVEHOST_UPDATE"); break;
-        case 1302:
-            return gm_strdup("NEBTYPE_ADAPTIVESERVICE_UPDATE"); break;
-        case 1303:
-            return gm_strdup("NEBTYPE_ADAPTIVECONTACT_UPDATE"); break;
-        case 1400:
-            return gm_strdup("NEBTYPE_EXTERNALCOMMAND_START"); break;
-        case 1401:
-            return gm_strdup("NEBTYPE_EXTERNALCOMMAND_END"); break;
-        case 1500:
-            return gm_strdup("NEBTYPE_AGGREGATEDSTATUS_STARTDUMP"); break;
-        case 1501:
-            return gm_strdup("NEBTYPE_AGGREGATEDSTATUS_ENDDUMP"); break;
-        case 1600:
-            return gm_strdup("NEBTYPE_RETENTIONDATA_STARTLOAD"); break;
-        case 1601:
-            return gm_strdup("NEBTYPE_RETENTIONDATA_ENDLOAD"); break;
-        case 1602:
-            return gm_strdup("NEBTYPE_RETENTIONDATA_STARTSAVE"); break;
-        case 1603:
-            return gm_strdup("NEBTYPE_RETENTIONDATA_ENDSAVE"); break;
-        case 1700:
-            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_ADD"); break;
-        case 1701:
-            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_REMOVE"); break;
-        case 1702:
-            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_LOAD"); break;
-        case 1800:
-            return gm_strdup("NEBTYPE_STATECHANGE_START"); break;
-        case 1801:
-            return gm_strdup("NEBTYPE_STATECHANGE_END"); break;
-    }
-    return gm_strdup("UNKNOWN");
-}
-
-
-/* return human readable name for nebcallback */
-char * nebcallback2str(int i) {
-    switch(i) {
-        case 0:
-            return gm_strdup("NEBCALLBACK_RESERVED0"); break;
-        case 1:
-            return gm_strdup("NEBCALLBACK_RESERVED1"); break;
-        case 2:
-            return gm_strdup("NEBCALLBACK_RESERVED2"); break;
-        case 3:
-            return gm_strdup("NEBCALLBACK_RESERVED3"); break;
-        case 4:
-            return gm_strdup("NEBCALLBACK_RESERVED4"); break;
-        case 5:
-            return gm_strdup("NEBCALLBACK_RAW_DATA"); break;
-        case 6:
-            return gm_strdup("NEBCALLBACK_NEB_DATA"); break;
-        case 7:
-            return gm_strdup("NEBCALLBACK_PROCESS_DATA"); break;
-        case 8:
-            return gm_strdup("NEBCALLBACK_TIMED_EVENT_DATA"); break;
-        case 9:
-            return gm_strdup("NEBCALLBACK_LOG_DATA"); break;
-        case 10:
-            return gm_strdup("NEBCALLBACK_SYSTEM_COMMAND_DATA"); break;
-        case 11:
-            return gm_strdup("NEBCALLBACK_EVENT_HANDLER_DATA"); break;
-        case 12:
-            return gm_strdup("NEBCALLBACK_NOTIFICATION_DATA"); break;
-        case 13:
-            return gm_strdup("NEBCALLBACK_SERVICE_CHECK_DATA"); break;
-        case 14:
-            return gm_strdup("NEBCALLBACK_HOST_CHECK_DATA"); break;
-        case 15:
-            return gm_strdup("NEBCALLBACK_COMMENT_DATA"); break;
-        case 16:
-            return gm_strdup("NEBCALLBACK_DOWNTIME_DATA"); break;
-        case 17:
-            return gm_strdup("NEBCALLBACK_FLAPPING_DATA"); break;
-        case 18:
-            return gm_strdup("NEBCALLBACK_PROGRAM_STATUS_DATA"); break;
-        case 19:
-            return gm_strdup("NEBCALLBACK_HOST_STATUS_DATA"); break;
-        case 20:
-            return gm_strdup("NEBCALLBACK_SERVICE_STATUS_DATA"); break;
-        case 21:
-            return gm_strdup("NEBCALLBACK_ADAPTIVE_PROGRAM_DATA"); break;
-        case 22:
-            return gm_strdup("NEBCALLBACK_ADAPTIVE_HOST_DATA"); break;
-        case 23:
-            return gm_strdup("NEBCALLBACK_ADAPTIVE_SERVICE_DATA"); break;
-        case 24:
-            return gm_strdup("NEBCALLBACK_EXTERNAL_COMMAND_DATA"); break;
-        case 25:
-            return gm_strdup("NEBCALLBACK_AGGREGATED_STATUS_DATA"); break;
-        case 26:
-            return gm_strdup("NEBCALLBACK_RETENTION_DATA"); break;
-        case 27:
-            return gm_strdup("NEBCALLBACK_CONTACT_NOTIFICATION_DATA"); break;
-        case 28:
-            return gm_strdup("NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA"); break;
-        case 29:
-            return gm_strdup("NEBCALLBACK_ACKNOWLEDGEMENT_DATA"); break;
-        case 30:
-            return gm_strdup("NEBCALLBACK_STATE_CHANGE_DATA"); break;
-        case 31:
-            return gm_strdup("NEBCALLBACK_CONTACT_STATUS_DATA"); break;
-        case 32:
-            return gm_strdup("NEBCALLBACK_ADAPTIVE_CONTACT_DATA"); break;
-    }
-    return gm_strdup("UNKNOWN");
-}
-
-/* return human readable name for eventtype */
-char * eventtype2str(int i) {
-    switch(i) {
-        case 0:
-            return gm_strdup("EVENT_SERVICE_CHECK"); break;
-        case 1:
-            return gm_strdup("EVENT_COMMAND_CHECK"); break;
-        case 2:
-            return gm_strdup("EVENT_LOG_ROTATION"); break;
-        case 3:
-            return gm_strdup("EVENT_PROGRAM_SHUTDOWN"); break;
-        case 4:
-            return gm_strdup("EVENT_PROGRAM_RESTART"); break;
-        case 5:
-            return gm_strdup("EVENT_CHECK_REAPER"); break;
-        case 6:
-            return gm_strdup("EVENT_ORPHAN_CHECK"); break;
-        case 7:
-            return gm_strdup("EVENT_RETENTION_SAVE"); break;
-        case 8:
-            return gm_strdup("EVENT_STATUS_SAVE"); break;
-        case 9:
-            return gm_strdup("EVENT_SCHEDULED_DOWNTIME"); break;
-        case 10:
-            return gm_strdup("EVENT_SFRESHNESS_CHECK"); break;
-        case 11:
-            return gm_strdup("EVENT_EXPIRE_DOWNTIME"); break;
-        case 12:
-            return gm_strdup("EVENT_HOST_CHECK"); break;
-        case 13:
-            return gm_strdup("EVENT_HFRESHNESS_CHECK"); break;
-        case 14:
-            return gm_strdup("EVENT_RESCHEDULE_CHECKS"); break;
-        case 15:
-            return gm_strdup("EVENT_EXPIRE_COMMENT"); break;
-        case 16:
-            return gm_strdup("EVENT_CHECK_PROGRAM_UPDATE"); break;
-        case 98:
-            return gm_strdup("EVENT_SLEEP"); break;
-        case 99:
-            return gm_strdup("EVENT_USER_FUNCTION"); break;
-    }
-    return gm_strdup("UNKNOWN");
-}
-
 /* generic logger function */
 void gm_log( int lvl, const char *text, ... ) {
     FILE * fp       = NULL;
