@@ -55,13 +55,14 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "2.1.5"
+#define GM_VERSION                 "3.0.0"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
 #define GM_BUFFERSIZE               65536
 #define GM_MAX_OUTPUT             1048576   /* limit plugin output size to 1mb */
 #define GM_LISTSIZE                   512
 #define GM_NEBTYPESSIZE                33   /* maximum number of neb types */
+#define GM_MAX_HOST_ADDRESS_LENGTH    256   /* max size of a host address */
 
 #define GM_MIN_LIB_GEARMAN_VERSION   0.14
 #define GM_SERVER_DEFAULT_PORT       4730
@@ -70,8 +71,6 @@
 #define GM_ERROR                        1
 #define GM_NO_EPN                      -1
 
-#define GM_PERFDATA_ALL                 2
-
 #define GM_EXIT_UNKNOWN               768   /* results in exit code 3 after processed by WEXITSTATUS() */
 
 /* log modes */
@@ -204,6 +203,9 @@ typedef struct mod_gm_opt_struct {
     int            result_workers;                          /**< number of result worker threads started */
     int            perfdata;                                /**< flag whether perfdata will be distributed or not */
     int            perfdata_mode;                           /**< flag whether perfdata will be sent with/without uniq set */
+    int            perfdata_send_all;                       /**< flag whether perfdata will be sent to all queues */
+    char         * perfdata_queues_list[GM_LISTSIZE];       /**< list of perfdata queue names */
+    int            perfdata_queues_num;                     /**< number of perfdata queues */
     char         * local_hostgroups_list[GM_LISTSIZE];      /**< list of hostgroups which will not be distributed */
     int            local_hostgroups_num;                    /**< number of elements in local_hostgroups_list */
     char         * local_servicegroups_list[GM_LISTSIZE];   /**< list of group  which will not be distributed */
@@ -259,6 +261,7 @@ typedef struct mod_gm_opt_struct {
     struct timeval finishtime;                              /**< time when the check finished */
     int            has_latency;                             /**< flag when latency is set */
     struct timeval latency;                                 /**< latency for this result */
+    int            gearman_connection_timeout;              /**< timeout on job submission */
 } mod_gm_opt_t;
 
 
