@@ -30,15 +30,36 @@
 NEB_API_VERSION( CURRENT_NEB_API_VERSION )
 
 /* import some global variables */
+#ifdef USENAGIOS3
+extern int            event_broker_options;
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
 extern unsigned long  event_broker_options;
+#ifdef USENAEMON
+#define my_free nm_free
+#endif
+#endif
 extern int            currently_running_host_checks;
 extern int            currently_running_service_checks;
 extern int            service_check_timeout;
 extern int            host_check_timeout;
+#ifdef USENAGIOS
+extern timed_event  * event_list_low;
+extern timed_event  * event_list_low_tail;
+#endif
 extern int            process_performance_data;
+#ifdef USENAGIOS3
+extern check_result   check_result_info;
+extern check_result * check_result_list;
+#endif
 
 /* global variables */
+#ifdef USENAGIOS3
+static check_result * mod_gm_result_list = 0;
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
 static objectlist * mod_gm_result_list = 0;
+#endif
 static pthread_mutex_t mod_gm_result_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 void *gearman_module_handle=NULL;
 gearman_client_st client;
@@ -59,9 +80,20 @@ static int   handle_perfdata(int e, void *);
 static int   handle_export(int e, void *);
 static void  set_target_queue( host *, service * );
 static int   handle_process_events( int, void * );
+#ifdef USENAGIOS
 static int   handle_timed_events( int, void * );
+#endif
 static void  start_threads(void);
+#ifdef USENAGIOS3
+static check_result * merge_result_lists(check_result * lista, check_result * listb);
+static void move_results_to_core_3x(void);
+#endif
+#ifdef USENAGIOS4
 static void move_results_to_core(void);
+#endif
+#ifdef USENAEMON
+static void move_results_to_core(struct nm_event_execution_properties *evprop);
+#endif
 
 int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     int i;
@@ -149,7 +181,12 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
 
     /* register callback for process event where everything else starts */
     neb_register_callback( NEBCALLBACK_PROCESS_DATA, gearman_module_handle, 0, handle_process_events );
+#ifdef USENAGIOS
     neb_register_callback( NEBCALLBACK_TIMED_EVENT_DATA, gearman_module_handle, 0, handle_timed_events );
+#endif
+#ifdef USENAEMON
+    schedule_event(1, move_results_to_core, NULL);
+#endif
 
     /* register export callbacks */
     for(i=0;i<GM_NEBTYPESSIZE;i++) {
@@ -253,6 +290,7 @@ int nebmodule_deinit( int flags, int reason ) {
 
 
 /* handle timed events */
+#ifdef USENAGIOS
 static int handle_timed_events( int event_type, void *data ) {
     nebstruct_timed_event_data * ted = (nebstruct_timed_event_data *)data;
 
@@ -266,22 +304,58 @@ static int handle_timed_events( int event_type, void *data ) {
 
     gm_log( GM_LOG_TRACE, "handle_timed_events(%i, data)\n", event_type, ted->event_type );
 
+#ifdef USENAGIOS3
+    move_results_to_core_3x();
+#endif
+#ifdef USENAGIOS4
     move_results_to_core();
+#endif
 
     return NEB_OK;
 }
+#endif
 
+#ifdef USENAGIOS3
+/* merge results with core */
+static check_result * merge_result_lists(check_result * lista, check_result * listb) {
+    check_result * result = 0;
 
-/* insert results list into core */
+    check_result ** iter;
+    for (iter = &result; lista && listb; iter = &(*iter)->next) {
+        if (mod_gm_time_compare(&lista->finish_time, &listb->finish_time) <= 0) {
+            *iter = lista; lista = lista->next;
+        } else {
+            *iter = listb; listb = listb->next;
+        }
+    }
+
+    *iter = lista? lista: listb;
+
+    return result;
+}
+#endif
+
+/* insert results list into naemon/nagios4 core */
+#if defined(USENAEMON) || defined(USENAGIOS4)
+#ifdef USENAEMON
+static void move_results_to_core(struct nm_event_execution_properties *evprop) {
+#endif
+#ifdef USENAGIOS4
 static void move_results_to_core() {
+#endif
+#ifdef USENAEMON
+    if(evprop->execution_type == EVENT_EXEC_NORMAL) {
+#endif
     /* safely save off currently local list */
     pthread_mutex_lock(&mod_gm_result_list_mutex);
 
     objectlist *tmp_list = NULL;
-    for(mod_gm_result_list; mod_gm_result_list; mod_gm_result_list = mod_gm_result_list->next) {
+    for( ; mod_gm_result_list; mod_gm_result_list = mod_gm_result_list->next) {
         free(tmp_list);
         process_check_result(mod_gm_result_list->object_ptr);
         free_check_result(mod_gm_result_list->object_ptr);
+        check_result *info = mod_gm_result_list->object_ptr;
+        my_free(info->source);
         free(mod_gm_result_list->object_ptr);
         tmp_list = mod_gm_result_list;
     }
@@ -289,16 +363,57 @@ static void move_results_to_core() {
     free(tmp_list);
 
     pthread_mutex_unlock(&mod_gm_result_list_mutex);
+#ifdef USENAEMON
+        schedule_event(1, move_results_to_core, NULL);
+    }
+#endif
 }
+#endif
 
+/* insert results list into nagios 3 core */
+#ifdef USENAGIOS3
+static void move_results_to_core_3x() {
+   check_result * local;
+
+   /* safely save off currently local list */
+   pthread_mutex_lock(&mod_gm_result_list_mutex);
+   local = mod_gm_result_list;
+   mod_gm_result_list = 0;
+   pthread_mutex_unlock(&mod_gm_result_list_mutex);
+
+   /* merge local into check_result_list, store in check_result_list */
+   check_result_list = merge_result_lists(local, check_result_list);
+}
+#endif
 
 /* add list to gearman result list */
+#if defined(USENAEMON) || defined(USENAGIOS4)
 void mod_gm_add_result_to_list(check_result * newcr) {
     pthread_mutex_lock(&mod_gm_result_list_mutex);
     add_object_to_objectlist(&mod_gm_result_list, newcr);
     pthread_mutex_unlock(&mod_gm_result_list_mutex);
 }
+#endif
 
+/* add list to gearman result list */
+#ifdef USENAGIOS3
+void mod_gm_add_result_to_list(check_result * newcr) {
+   check_result ** curp;
+
+   assert(newcr);
+
+   pthread_mutex_lock(&mod_gm_result_list_mutex);
+
+   for (curp = &mod_gm_result_list; *curp; curp = &(*curp)->next)
+      if (mod_gm_time_compare(&(*curp)->finish_time, &newcr->finish_time) >= 0)
+         break;
+
+   newcr->next = *curp;
+   *curp = newcr;
+
+   pthread_mutex_unlock(&mod_gm_result_list_mutex);
+}
+#endif
 
 /* handle process events */
 static int handle_process_events( int event_type, void *data ) {
@@ -456,11 +571,16 @@ static int handle_host_check( int event_type, void *data ) {
     char *raw_command=NULL;
     char *processed_command=NULL;
     host * hst;
+#ifdef USENAGIOS
     check_result * chk_result;
     int check_options;
+#endif
     struct timeval core_time;
     struct tm next_check;
     char buffer1[GM_BUFFERSIZE];
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    nagios_macros mac;
+#endif
 
     gettimeofday(&core_time,NULL);
 
@@ -501,35 +621,67 @@ static int handle_host_check( int event_type, void *data ) {
      * we have to do some host check logic here
      * taken from checks.c:
      */
+#ifdef USENAGIOS
     /* clear check options - we don't want old check options retained */
     check_options = hst->check_options;
     hst->check_options = CHECK_OPTION_NONE;
+#endif
 
     /* unset the freshening flag, otherwise only the first freshness check would be run */
     hst->is_being_freshened=FALSE;
 
     /* adjust host check attempt */
+#ifdef USENAGIOS3
+    adjust_host_check_attempt_3x(hst,TRUE);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
     adjust_host_check_attempt(hst,TRUE);
+#endif
 
     temp_buffer[0]='\x0';
 
     /* grab the host macro variables */
+#ifdef USENAGIOS3
     clear_volatile_macros();
     grab_host_macros(hst);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    memset(&mac, 0, sizeof(mac));
+    clear_volatile_macros_r(&mac);
+    grab_host_macros_r(&mac, hst);
+#endif
 
     /* get the raw command line */
-    get_raw_command_line(hst->check_command_ptr,hst->check_command,&raw_command,0);
+#ifdef USENAGIOS3
+    get_raw_command_line(hst->check_command_ptr,hst->host_check_command,&raw_command,0);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    get_raw_command_line_r(&mac, hst->check_command_ptr, hst->check_command, &raw_command, 0);
+#endif
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for host '%s' was NULL - aborting.\n",hst->name );
         return NEBERROR_CALLBACKCANCEL;
     }
 
     /* process any macros contained in the argument */
+#ifdef USENAGIOS3
     process_macros(raw_command,&processed_command,0);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    process_macros_r(&mac, raw_command, &processed_command, 0);
+#endif
     if(processed_command==NULL){
         gm_log( GM_LOG_ERROR, "Processed check command for host '%s' was NULL - aborting.\n",hst->name);
         return NEBERROR_CALLBACKCANCEL;
     }
+#if defined(USENAEMON)
+    /* naemon sends unescaped newlines from ex.: the LONGPLUGINOUTPUT macro, so we have to escape
+     * them ourselves: https://github.com/naemon/naemon-core/issues/153 */
+    char *tmp = replace_str(processed_command, "\n", "\\n");
+    free(processed_command);
+    processed_command = replace_str(tmp, "\n", "\\n");
+    free(tmp);
+#endif
 
     /* log latency */
     if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
@@ -587,6 +739,7 @@ static int handle_host_check( int event_type, void *data ) {
     my_free(processed_command);
 
     /* orphaned check - submit fake result to mark host as orphaned */
+#ifdef USENAGIOS
     if(mod_gm_opt->orphan_host_checks == GM_ENABLED && check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "host check for %s orphaned\n", hst->name );
         if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
@@ -609,6 +762,7 @@ static int handle_host_check( int event_type, void *data ) {
         mod_gm_add_result_to_list( chk_result );
         chk_result = NULL;
     }
+#endif
 
     /* tell naemon to not execute */
     gm_log( GM_LOG_TRACE, "handle_host_check() finished successfully -> %d\n", NEBERROR_CALLBACKOVERRIDE );
@@ -624,10 +778,15 @@ static int handle_svc_check( int event_type, void *data ) {
     char *processed_command=NULL;
     nebstruct_service_check_data * svcdata;
     int prio = GM_JOB_PRIO_LOW;
+#ifdef USENAGIOS
     check_result * chk_result;
+#endif
     struct timeval core_time;
     struct tm next_check;
     char buffer1[GM_BUFFERSIZE];
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    nagios_macros mac;
+#endif
 
     gettimeofday(&core_time,NULL);
 
@@ -676,24 +835,49 @@ static int handle_svc_check( int event_type, void *data ) {
     svc->is_being_freshened=FALSE;
 
     /* grab the host and service macro variables */
+#ifdef USENAGIOS3
     clear_volatile_macros();
     grab_host_macros(hst);
     grab_service_macros(svc);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    memset(&mac, 0, sizeof(mac));
+    clear_volatile_macros_r(&mac);
+    grab_host_macros_r(&mac, hst);
+    grab_service_macros_r(&mac, svc);
+#endif
 
     /* get the raw command line */
-    get_raw_command_line(svc->check_command_ptr,svc->check_command,&raw_command,0);
+#ifdef USENAGIOS3
+    get_raw_command_line(svc->check_command_ptr,svc->service_check_command,&raw_command,0);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    get_raw_command_line_r(&mac, svc->check_command_ptr, svc->check_command, &raw_command, 0);
+#endif
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for service '%s' on host '%s' was NULL - aborting.\n", svc->description, svc->host_name );
         return NEBERROR_CALLBACKCANCEL;
     }
 
     /* process any macros contained in the argument */
-    process_macros(raw_command, &processed_command, 0);
+#ifdef USENAGIOS3
+    process_macros(raw_command,&processed_command,0);
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
+    process_macros_r(&mac, raw_command, &processed_command, 0);
+#endif
     if(processed_command==NULL) {
         gm_log( GM_LOG_ERROR, "Processed check command for service '%s' on host '%s' was NULL - aborting.\n", svc->description, svc->host_name);
         my_free(raw_command);
         return NEBERROR_CALLBACKCANCEL;
     }
+#if defined(USENAEMON)
+    /* naemon sends unescaped newlines from ex.: the LONGPLUGINOUTPUT macro, so we have to escape
+     * them ourselves: https://github.com/naemon/naemon-core/issues/153 */
+    char *tmp = replace_str(processed_command, "\n", "\\n");
+    free(processed_command);
+    processed_command = tmp;
+#endif
 
     /* log latency */
     if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
@@ -726,7 +910,12 @@ static int handle_svc_check( int event_type, void *data ) {
     snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", svcdata->host_name, svcdata->service_description);
 
     /* execute forced checks with high prio as they are propably user requested */
+#ifdef USENAGIOS3
+    if(check_result_info.check_options & CHECK_OPTION_FORCE_EXECUTION)
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
     if(svc->check_options & CHECK_OPTION_FORCE_EXECUTION)
+#endif
         prio = GM_JOB_PRIO_HIGH;
 
     if(add_job_to_queue( &client,
@@ -760,6 +949,7 @@ static int handle_svc_check( int event_type, void *data ) {
     my_free(processed_command);
 
     /* orphaned check - submit fake result to mark service as orphaned */
+#ifdef USENAGIOS
     if(mod_gm_opt->orphan_service_checks == GM_ENABLED && svc->check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "service check for %s - %s orphaned\n", svc->host_name, svc->description );
         if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
@@ -783,6 +973,7 @@ static int handle_svc_check( int event_type, void *data ) {
         mod_gm_add_result_to_list( chk_result );
         chk_result = NULL;
     }
+#endif
 
     /* tell naemon to not execute */
     gm_log( GM_LOG_TRACE, "handle_svc_check() finished successfully -> %d\n", NEBERROR_CALLBACKOVERRIDE );
@@ -816,15 +1007,15 @@ static int read_arguments( const char *args_orig ) {
 
     verify = verify_options(mod_gm_opt);
 
-    if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
-        dumpconfig(mod_gm_opt, GM_NEB_MODE);
-    }
-
     /* read keyfile */
     if(mod_gm_opt->keyfile != NULL && read_keyfile(mod_gm_opt) != GM_OK) {
         errors++;
     }
 
+    if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
+        dumpconfig(mod_gm_opt, GM_NEB_MODE);
+    }
+
     free(args_c);
 
     if(errors > 0) {
@@ -917,7 +1108,7 @@ static void set_target_queue( host *hst, service *svc ) {
     if ( svc ) {
         while ( mod_gm_opt->local_servicegroups_list[x] != NULL ) {
             servicegroup * temp_servicegroup = find_servicegroup( mod_gm_opt->local_servicegroups_list[x] );
-            if ( is_service_member_of_servicegroup( temp_servicegroup,svc )==TRUE ) {
+            if ( temp_servicegroup != NULL && is_service_member_of_servicegroup( temp_servicegroup,svc )==TRUE ) {
                 gm_log( GM_LOG_TRACE, "service is member of local servicegroup: %s\n", mod_gm_opt->local_servicegroups_list[x] );
                 return;
             }
@@ -929,7 +1120,7 @@ static void set_target_queue( host *hst, service *svc ) {
     x = 0;
     while ( mod_gm_opt->local_hostgroups_list[x] != NULL ) {
         hostgroup * temp_hostgroup = find_hostgroup( mod_gm_opt->local_hostgroups_list[x] );
-        if ( is_host_member_of_hostgroup( temp_hostgroup,hst )==TRUE ) {
+        if ( temp_hostgroup != NULL && is_host_member_of_hostgroup( temp_hostgroup,hst )==TRUE ) {
             gm_log( GM_LOG_TRACE, "server is member of local hostgroup: %s\n", mod_gm_opt->local_hostgroups_list[x] );
             return;
         }
@@ -941,7 +1132,7 @@ static void set_target_queue( host *hst, service *svc ) {
     if ( svc ) {
         while ( mod_gm_opt->servicegroups_list[x] != NULL ) {
             servicegroup * temp_servicegroup = find_servicegroup( mod_gm_opt->servicegroups_list[x] );
-            if ( is_service_member_of_servicegroup( temp_servicegroup,svc )==TRUE ) {
+            if ( temp_servicegroup != NULL && is_service_member_of_servicegroup( temp_servicegroup,svc )==TRUE ) {
                 gm_log( GM_LOG_TRACE, "service is member of servicegroup: %s\n", mod_gm_opt->servicegroups_list[x] );
                 snprintf( target_queue, GM_BUFFERSIZE-1, "servicegroup_%s", mod_gm_opt->servicegroups_list[x] );
                 return;
@@ -954,7 +1145,7 @@ static void set_target_queue( host *hst, service *svc ) {
     x = 0;
     while ( mod_gm_opt->hostgroups_list[x] != NULL ) {
         hostgroup * temp_hostgroup = find_hostgroup( mod_gm_opt->hostgroups_list[x] );
-        if ( is_host_member_of_hostgroup( temp_hostgroup,hst )==TRUE ) {
+        if ( temp_hostgroup != NULL && is_host_member_of_hostgroup( temp_hostgroup,hst )==TRUE ) {
             gm_log( GM_LOG_TRACE, "server is member of hostgroup: %s\n", mod_gm_opt->hostgroups_list[x] );
             snprintf( target_queue, GM_BUFFERSIZE-1, "hostgroup_%s", mod_gm_opt->hostgroups_list[x] );
             return;
@@ -1001,7 +1192,10 @@ int handle_perfdata(int event_type, void *data) {
     host *hst        = NULL;
     service *svc     = NULL;
     int has_perfdata = FALSE;
+    int i;
+#if defined(USENAEMON) || defined(USENAGIOS4)
     char *perf_data;
+#endif
 
     gm_log( GM_LOG_TRACE, "handle_perfdata(%d)\n", event_type );
     if(process_performance_data == 0) {
@@ -1021,7 +1215,7 @@ int handle_perfdata(int event_type, void *data) {
                 }
 
                 hst = (host *) hostchkdata->object_ptr;
-                if(hst->process_performance_data == 0 && mod_gm_opt->perfdata != GM_PERFDATA_ALL) {
+                if(hst->process_performance_data == 0 && mod_gm_opt->perfdata_send_all == GM_DISABLED) {
                     gm_log( GM_LOG_TRACE, "handle_perfdata() process_performance_data disabled for: %s\n", hst->name );
                     break;
                 }
@@ -1029,8 +1223,10 @@ int handle_perfdata(int event_type, void *data) {
                 uniq[0]='\x0';
                 snprintf( uniq,GM_BUFFERSIZE-1,"%s", hostchkdata->host_name);
 
+#if defined(USENAEMON) || defined(USENAGIOS4)
                 /* replace newlines with actual newlines */
                 perf_data = replace_str(hostchkdata->perf_data, "\\n", "\n");
+#endif
 
 
                 temp_buffer[0]='\x0';
@@ -1044,11 +1240,20 @@ int handle_perfdata(int event_type, void *data) {
                             "HOSTSTATETYPE::%d\n"
                             "HOSTINTERVAL::%f\n\n",
                             (int)hostchkdata->timestamp.tv_sec,
+#ifdef USENAGIOS3
+                            hostchkdata->host_name, hostchkdata->perf_data,
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
                             hostchkdata->host_name, perf_data,
+#endif
                             hostchkdata->command_name, hostchkdata->command_args,
                             hostchkdata->state, hostchkdata->state_type,
                             hst->check_interval);
                 has_perfdata = TRUE;
+#if defined(USENAEMON) || defined(USENAGIOS4)
+                free(perf_data);
+#endif
+
             }
             break;
 
@@ -1062,7 +1267,7 @@ int handle_perfdata(int event_type, void *data) {
 
                 /* find the naemon service object for this service */
                 svc = (service *) srvchkdata->object_ptr;
-                if(svc->process_performance_data == 0 && mod_gm_opt->perfdata != GM_PERFDATA_ALL) {
+                if(svc->process_performance_data == 0 && mod_gm_opt->perfdata_send_all == GM_DISABLED) {
                     gm_log( GM_LOG_TRACE, "handle_perfdata() process_performance_data disabled for: %s - %s\n", svc->host_name, svc->description );
                     break;
                 }
@@ -1070,8 +1275,10 @@ int handle_perfdata(int event_type, void *data) {
                 uniq[0]='\x0';
                 snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", srvchkdata->host_name, srvchkdata->service_description);
 
+#if defined(USENAEMON) || defined(USENAGIOS4)
                 /* replace newlines with actual newlines */
                 perf_data = replace_str(srvchkdata->perf_data, "\\n", "\n");
+#endif
 
                 temp_buffer[0]='\x0';
                 snprintf( temp_buffer,GM_BUFFERSIZE-1,
@@ -1086,11 +1293,19 @@ int handle_perfdata(int event_type, void *data) {
                             "SERVICEINTERVAL::%f\n\n",
                             (int)srvchkdata->timestamp.tv_sec,
                             srvchkdata->host_name, srvchkdata->service_description,
+#ifdef USENAGIOS3
+                            srvchkdata->perf_data, svc->service_check_command,
+#endif
+#if defined(USENAEMON) || defined(USENAGIOS4)
                             perf_data, svc->check_command,
+#endif
                             srvchkdata->state, srvchkdata->state_type,
                             svc->check_interval);
                 temp_buffer[GM_BUFFERSIZE-1]='\x0';
                 has_perfdata = TRUE;
+#if defined(USENAEMON) || defined(USENAGIOS4)
+                free(perf_data);
+#endif
             }
             break;
 
@@ -1099,21 +1314,24 @@ int handle_perfdata(int event_type, void *data) {
     }
 
     if(has_perfdata == TRUE) {
-        /* add our job onto the queue */
-        if(add_job_to_queue( &client,
-                             mod_gm_opt->server_list,
-                             GM_PERFDATA_QUEUE,
-                             (mod_gm_opt->perfdata_mode == GM_PERFDATA_OVERWRITE ? uniq : NULL),
-                             temp_buffer,
-                             GM_JOB_PRIO_NORMAL,
-                             GM_DEFAULT_JOB_RETRIES,
-                             mod_gm_opt->transportmode,
-                             TRUE
-                            ) == GM_OK) {
-            gm_log( GM_LOG_TRACE, "handle_perfdata() finished successfully\n" );
-        }
-        else {
-            gm_log( GM_LOG_TRACE, "handle_perfdata() finished unsuccessfully\n" );
+        for (i = 0; i < mod_gm_opt->perfdata_queues_num; i++) {
+            char *perfdata_queue = mod_gm_opt->perfdata_queues_list[i];
+            /* add our job onto the queue */
+            if(add_job_to_queue( &client,
+                                 mod_gm_opt->server_list,
+                                 perfdata_queue,
+                                 (mod_gm_opt->perfdata_mode == GM_PERFDATA_OVERWRITE ? uniq : NULL),
+                                 temp_buffer,
+                                 GM_JOB_PRIO_NORMAL,
+                                 GM_DEFAULT_JOB_RETRIES,
+                                 mod_gm_opt->transportmode,
+                                 TRUE
+                                ) == GM_OK) {
+                gm_log( GM_LOG_TRACE, "handle_perfdata() successfully added data to %s\n", perfdata_queue );
+            }
+            else {
+                gm_log( GM_LOG_TRACE, "handle_perfdata() failed to add data to %s\n", perfdata_queue );
+            }
         }
     }
 
@@ -1259,6 +1477,266 @@ int handle_export(int callback_type, void *data) {
 
 /* core log wrapper */
 void write_core_log(char *data) {
+#ifdef USENAEMON
     nm_log( NSLOG_INFO_MESSAGE, data );
+#endif
+#ifdef USENAGIOS
+    write_to_all_logs( data, NSLOG_INFO_MESSAGE );
+#endif
     return;
 }
+
+
+/* return human readable name for neb type */
+char * nebtype2str(int i) {
+    switch(i) {
+        case NEBTYPE_NONE:
+            return gm_strdup("NEBTYPE_NONE"); break;
+        case NEBTYPE_HELLO:
+            return gm_strdup("NEBTYPE_HELLO"); break;
+        case NEBTYPE_GOODBYE:
+            return gm_strdup("NEBTYPE_GOODBYE"); break;
+        case NEBTYPE_INFO:
+            return gm_strdup("NEBTYPE_INFO"); break;
+        case NEBTYPE_PROCESS_START:
+            return gm_strdup("NEBTYPE_PROCESS_START"); break;
+        case NEBTYPE_PROCESS_DAEMONIZE:
+            return gm_strdup("NEBTYPE_PROCESS_DAEMONIZE"); break;
+        case NEBTYPE_PROCESS_RESTART:
+            return gm_strdup("NEBTYPE_PROCESS_RESTART"); break;
+        case NEBTYPE_PROCESS_SHUTDOWN:
+            return gm_strdup("NEBTYPE_PROCESS_SHUTDOWN"); break;
+        case NEBTYPE_PROCESS_PRELAUNCH:
+            return gm_strdup("NEBTYPE_PROCESS_PRELAUNCH"); break;
+        case NEBTYPE_PROCESS_EVENTLOOPSTART:
+            return gm_strdup("NEBTYPE_PROCESS_EVENTLOOPSTART"); break;
+        case NEBTYPE_PROCESS_EVENTLOOPEND:
+            return gm_strdup("NEBTYPE_PROCESS_EVENTLOOPEND"); break;
+        case NEBTYPE_TIMEDEVENT_ADD:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_ADD"); break;
+        case NEBTYPE_TIMEDEVENT_REMOVE:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_REMOVE"); break;
+        case NEBTYPE_TIMEDEVENT_EXECUTE:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_EXECUTE"); break;
+        case NEBTYPE_TIMEDEVENT_DELAY:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_DELAY"); break;
+        case NEBTYPE_TIMEDEVENT_SKIP:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_SKIP"); break;
+        case NEBTYPE_TIMEDEVENT_SLEEP:
+            return gm_strdup("NEBTYPE_TIMEDEVENT_SLEEP"); break;
+        case NEBTYPE_LOG_DATA:
+            return gm_strdup("NEBTYPE_LOG_DATA"); break;
+        case NEBTYPE_LOG_ROTATION:
+            return gm_strdup("NEBTYPE_LOG_ROTATION"); break;
+        case NEBTYPE_SYSTEM_COMMAND_START:
+            return gm_strdup("NEBTYPE_SYSTEM_COMMAND_START"); break;
+        case NEBTYPE_SYSTEM_COMMAND_END:
+            return gm_strdup("NEBTYPE_SYSTEM_COMMAND_END"); break;
+        case NEBTYPE_EVENTHANDLER_START:
+            return gm_strdup("NEBTYPE_EVENTHANDLER_START"); break;
+        case NEBTYPE_EVENTHANDLER_END:
+            return gm_strdup("NEBTYPE_EVENTHANDLER_END"); break;
+        case NEBTYPE_NOTIFICATION_START:
+            return gm_strdup("NEBTYPE_NOTIFICATION_START"); break;
+        case NEBTYPE_NOTIFICATION_END:
+            return gm_strdup("NEBTYPE_NOTIFICATION_END"); break;
+        case NEBTYPE_CONTACTNOTIFICATION_START:
+            return gm_strdup("NEBTYPE_CONTACTNOTIFICATION_START"); break;
+        case NEBTYPE_CONTACTNOTIFICATION_END:
+            return gm_strdup("NEBTYPE_CONTACTNOTIFICATION_END"); break;
+        case NEBTYPE_CONTACTNOTIFICATIONMETHOD_START:
+            return gm_strdup("NEBTYPE_CONTACTNOTIFICATIONMETHOD_START"); break;
+        case NEBTYPE_CONTACTNOTIFICATIONMETHOD_END:
+            return gm_strdup("NEBTYPE_CONTACTNOTIFICATIONMETHOD_END"); break;
+        case NEBTYPE_SERVICECHECK_INITIATE:
+            return gm_strdup("NEBTYPE_SERVICECHECK_INITIATE"); break;
+        case NEBTYPE_SERVICECHECK_PROCESSED:
+            return gm_strdup("NEBTYPE_SERVICECHECK_PROCESSED"); break;
+        case NEBTYPE_SERVICECHECK_RAW_START:
+            return gm_strdup("NEBTYPE_SERVICECHECK_RAW_START"); break;
+        case NEBTYPE_SERVICECHECK_RAW_END:
+            return gm_strdup("NEBTYPE_SERVICECHECK_RAW_END"); break;
+        case NEBTYPE_SERVICECHECK_ASYNC_PRECHECK:
+            return gm_strdup("NEBTYPE_SERVICECHECK_ASYNC_PRECHECK"); break;
+        case NEBTYPE_HOSTCHECK_INITIATE:
+            return gm_strdup("NEBTYPE_HOSTCHECK_INITIATE"); break;
+        case NEBTYPE_HOSTCHECK_PROCESSED:
+            return gm_strdup("NEBTYPE_HOSTCHECK_PROCESSED"); break;
+        case NEBTYPE_HOSTCHECK_RAW_START:
+            return gm_strdup("NEBTYPE_HOSTCHECK_RAW_START"); break;
+        case NEBTYPE_HOSTCHECK_RAW_END:
+            return gm_strdup("NEBTYPE_HOSTCHECK_RAW_END"); break;
+        case NEBTYPE_HOSTCHECK_ASYNC_PRECHECK:
+            return gm_strdup("NEBTYPE_HOSTCHECK_ASYNC_PRECHECK"); break;
+        case NEBTYPE_HOSTCHECK_SYNC_PRECHECK:
+            return gm_strdup("NEBTYPE_HOSTCHECK_SYNC_PRECHECK"); break;
+        case NEBTYPE_COMMENT_ADD:
+            return gm_strdup("NEBTYPE_COMMENT_ADD"); break;
+        case NEBTYPE_COMMENT_DELETE:
+            return gm_strdup("NEBTYPE_COMMENT_DELETE"); break;
+        case NEBTYPE_COMMENT_LOAD:
+            return gm_strdup("NEBTYPE_COMMENT_LOAD"); break;
+        case NEBTYPE_FLAPPING_START:
+            return gm_strdup("NEBTYPE_FLAPPING_START"); break;
+        case NEBTYPE_FLAPPING_STOP:
+            return gm_strdup("NEBTYPE_FLAPPING_STOP"); break;
+        case NEBTYPE_DOWNTIME_ADD:
+            return gm_strdup("NEBTYPE_DOWNTIME_ADD"); break;
+        case NEBTYPE_DOWNTIME_DELETE:
+            return gm_strdup("NEBTYPE_DOWNTIME_DELETE"); break;
+        case NEBTYPE_DOWNTIME_LOAD:
+            return gm_strdup("NEBTYPE_DOWNTIME_LOAD"); break;
+        case NEBTYPE_DOWNTIME_START:
+            return gm_strdup("NEBTYPE_DOWNTIME_START"); break;
+        case NEBTYPE_DOWNTIME_STOP:
+            return gm_strdup("NEBTYPE_DOWNTIME_STOP"); break;
+        case NEBTYPE_PROGRAMSTATUS_UPDATE:
+            return gm_strdup("NEBTYPE_PROGRAMSTATUS_UPDATE"); break;
+        case NEBTYPE_HOSTSTATUS_UPDATE:
+            return gm_strdup("NEBTYPE_HOSTSTATUS_UPDATE"); break;
+        case NEBTYPE_SERVICESTATUS_UPDATE:
+            return gm_strdup("NEBTYPE_SERVICESTATUS_UPDATE"); break;
+        case NEBTYPE_CONTACTSTATUS_UPDATE:
+            return gm_strdup("NEBTYPE_CONTACTSTATUS_UPDATE"); break;
+        case NEBTYPE_ADAPTIVEPROGRAM_UPDATE:
+            return gm_strdup("NEBTYPE_ADAPTIVEPROGRAM_UPDATE"); break;
+        case NEBTYPE_ADAPTIVEHOST_UPDATE:
+            return gm_strdup("NEBTYPE_ADAPTIVEHOST_UPDATE"); break;
+        case NEBTYPE_ADAPTIVESERVICE_UPDATE:
+            return gm_strdup("NEBTYPE_ADAPTIVESERVICE_UPDATE"); break;
+        case NEBTYPE_ADAPTIVECONTACT_UPDATE:
+            return gm_strdup("NEBTYPE_ADAPTIVECONTACT_UPDATE"); break;
+        case NEBTYPE_EXTERNALCOMMAND_START:
+            return gm_strdup("NEBTYPE_EXTERNALCOMMAND_START"); break;
+        case NEBTYPE_EXTERNALCOMMAND_END:
+            return gm_strdup("NEBTYPE_EXTERNALCOMMAND_END"); break;
+        case NEBTYPE_AGGREGATEDSTATUS_STARTDUMP:
+            return gm_strdup("NEBTYPE_AGGREGATEDSTATUS_STARTDUMP"); break;
+        case NEBTYPE_AGGREGATEDSTATUS_ENDDUMP:
+            return gm_strdup("NEBTYPE_AGGREGATEDSTATUS_ENDDUMP"); break;
+        case NEBTYPE_RETENTIONDATA_STARTLOAD:
+            return gm_strdup("NEBTYPE_RETENTIONDATA_STARTLOAD"); break;
+        case NEBTYPE_RETENTIONDATA_ENDLOAD:
+            return gm_strdup("NEBTYPE_RETENTIONDATA_ENDLOAD"); break;
+        case NEBTYPE_RETENTIONDATA_STARTSAVE:
+            return gm_strdup("NEBTYPE_RETENTIONDATA_STARTSAVE"); break;
+        case NEBTYPE_RETENTIONDATA_ENDSAVE:
+            return gm_strdup("NEBTYPE_RETENTIONDATA_ENDSAVE"); break;
+        case NEBTYPE_ACKNOWLEDGEMENT_ADD:
+            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_ADD"); break;
+        case NEBTYPE_ACKNOWLEDGEMENT_REMOVE:
+            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_REMOVE"); break;
+        case NEBTYPE_ACKNOWLEDGEMENT_LOAD:
+            return gm_strdup("NEBTYPE_ACKNOWLEDGEMENT_LOAD"); break;
+        case NEBTYPE_STATECHANGE_START:
+            return gm_strdup("NEBTYPE_STATECHANGE_START"); break;
+        case NEBTYPE_STATECHANGE_END:
+            return gm_strdup("NEBTYPE_STATECHANGE_END"); break;
+    }
+    return gm_strdup("UNKNOWN");
+}
+
+
+/* return human readable name for nebcallback */
+char * nebcallback2str(int i) {
+    switch(i) {
+        case NEBCALLBACK_PROCESS_DATA:
+            return gm_strdup("NEBCALLBACK_PROCESS_DATA"); break;
+        case NEBCALLBACK_TIMED_EVENT_DATA:
+            return gm_strdup("NEBCALLBACK_TIMED_EVENT_DATA"); break;
+        case NEBCALLBACK_LOG_DATA:
+            return gm_strdup("NEBCALLBACK_LOG_DATA"); break;
+        case NEBCALLBACK_SYSTEM_COMMAND_DATA:
+            return gm_strdup("NEBCALLBACK_SYSTEM_COMMAND_DATA"); break;
+        case NEBCALLBACK_EVENT_HANDLER_DATA:
+            return gm_strdup("NEBCALLBACK_EVENT_HANDLER_DATA"); break;
+        case NEBCALLBACK_NOTIFICATION_DATA:
+            return gm_strdup("NEBCALLBACK_NOTIFICATION_DATA"); break;
+        case NEBCALLBACK_SERVICE_CHECK_DATA:
+            return gm_strdup("NEBCALLBACK_SERVICE_CHECK_DATA"); break;
+        case NEBCALLBACK_HOST_CHECK_DATA:
+            return gm_strdup("NEBCALLBACK_HOST_CHECK_DATA"); break;
+        case NEBCALLBACK_COMMENT_DATA:
+            return gm_strdup("NEBCALLBACK_COMMENT_DATA"); break;
+        case NEBCALLBACK_DOWNTIME_DATA:
+            return gm_strdup("NEBCALLBACK_DOWNTIME_DATA"); break;
+        case NEBCALLBACK_FLAPPING_DATA:
+            return gm_strdup("NEBCALLBACK_FLAPPING_DATA"); break;
+        case NEBCALLBACK_PROGRAM_STATUS_DATA:
+            return gm_strdup("NEBCALLBACK_PROGRAM_STATUS_DATA"); break;
+        case NEBCALLBACK_HOST_STATUS_DATA:
+            return gm_strdup("NEBCALLBACK_HOST_STATUS_DATA"); break;
+        case NEBCALLBACK_SERVICE_STATUS_DATA:
+            return gm_strdup("NEBCALLBACK_SERVICE_STATUS_DATA"); break;
+        case NEBCALLBACK_ADAPTIVE_PROGRAM_DATA:
+            return gm_strdup("NEBCALLBACK_ADAPTIVE_PROGRAM_DATA"); break;
+        case NEBCALLBACK_ADAPTIVE_HOST_DATA:
+            return gm_strdup("NEBCALLBACK_ADAPTIVE_HOST_DATA"); break;
+        case NEBCALLBACK_ADAPTIVE_SERVICE_DATA:
+            return gm_strdup("NEBCALLBACK_ADAPTIVE_SERVICE_DATA"); break;
+        case NEBCALLBACK_EXTERNAL_COMMAND_DATA:
+            return gm_strdup("NEBCALLBACK_EXTERNAL_COMMAND_DATA"); break;
+        case NEBCALLBACK_AGGREGATED_STATUS_DATA:
+            return gm_strdup("NEBCALLBACK_AGGREGATED_STATUS_DATA"); break;
+        case NEBCALLBACK_RETENTION_DATA:
+            return gm_strdup("NEBCALLBACK_RETENTION_DATA"); break;
+        case NEBCALLBACK_CONTACT_NOTIFICATION_DATA:
+            return gm_strdup("NEBCALLBACK_CONTACT_NOTIFICATION_DATA"); break;
+        case NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA:
+            return gm_strdup("NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA"); break;
+        case NEBCALLBACK_ACKNOWLEDGEMENT_DATA:
+            return gm_strdup("NEBCALLBACK_ACKNOWLEDGEMENT_DATA"); break;
+        case NEBCALLBACK_STATE_CHANGE_DATA:
+            return gm_strdup("NEBCALLBACK_STATE_CHANGE_DATA"); break;
+        case NEBCALLBACK_CONTACT_STATUS_DATA:
+            return gm_strdup("NEBCALLBACK_CONTACT_STATUS_DATA"); break;
+        case NEBCALLBACK_ADAPTIVE_CONTACT_DATA:
+            return gm_strdup("NEBCALLBACK_ADAPTIVE_CONTACT_DATA"); break;
+    }
+    return gm_strdup("UNKNOWN");
+}
+
+/* return human readable name for eventtype */
+char * eventtype2str(int i) {
+    switch(i) {
+        case 0:
+            return gm_strdup("EVENT_SERVICE_CHECK"); break;
+        case 1:
+            return gm_strdup("EVENT_COMMAND_CHECK"); break;
+        case 2:
+            return gm_strdup("EVENT_LOG_ROTATION"); break;
+        case 3:
+            return gm_strdup("EVENT_PROGRAM_SHUTDOWN"); break;
+        case 4:
+            return gm_strdup("EVENT_PROGRAM_RESTART"); break;
+        case 5:
+            return gm_strdup("EVENT_CHECK_REAPER"); break;
+        case 6:
+            return gm_strdup("EVENT_ORPHAN_CHECK"); break;
+        case 7:
+            return gm_strdup("EVENT_RETENTION_SAVE"); break;
+        case 8:
+            return gm_strdup("EVENT_STATUS_SAVE"); break;
+        case 9:
+            return gm_strdup("EVENT_SCHEDULED_DOWNTIME"); break;
+        case 10:
+            return gm_strdup("EVENT_SFRESHNESS_CHECK"); break;
+        case 11:
+            return gm_strdup("EVENT_EXPIRE_DOWNTIME"); break;
+        case 12:
+            return gm_strdup("EVENT_HOST_CHECK"); break;
+        case 13:
+            return gm_strdup("EVENT_HFRESHNESS_CHECK"); break;
+        case 14:
+            return gm_strdup("EVENT_RESCHEDULE_CHECKS"); break;
+        case 15:
+            return gm_strdup("EVENT_EXPIRE_COMMENT"); break;
+        case 16:
+            return gm_strdup("EVENT_CHECK_PROGRAM_UPDATE"); break;
+        case 98:
+            return gm_strdup("EVENT_SLEEP"); break;
+        case 99:
+            return gm_strdup("EVENT_USER_FUNCTION"); break;
+    }
+    return gm_strdup("UNKNOWN");
+}
